==> uniprot_id_mapper-1.1.1-py3-none-any.whl: UniProtMapper/interface.py <==
# -*- coding: utf-8 -*-
"""Holds the implementation of the base class for the UniProt id-mapping REST API."""

import re
import time
from abc import ABC
from urllib.parse import parse_qs, urlparse

import requests
from requests.adapters import HTTPAdapter, Retry

from .utils import decode_results

"""
Several methods were either taken or adapted from the Python example for the
UniProt ID mapping RESTful API documentation. The original code can be found at:
https://www.uniprot.org/help/id_mapping#submitting-an-id-mapping-job
"""


class BaseUniProt(ABC):
    """Implementation of the base class for the UniProt REST API."""

    def __init__(
        self,
        pooling_interval=3,
        total_retries=5,
        backoff_factor=0.25,
        api_url="https://rest.uniprot.org",
    ) -> None:
        """Initialize the class. This will set up the session and retry mechanism.

        Args:
            pooling_interval: The interval in seconds between polling the API.
                Defaults to 3.
            total_retries: The total number of retries to attempt. Defaults to 5.
            backoff_factor: The backoff factor to use when retrying. Defaults to 0.25.
            api_url: The url for the REST API. Defaults to "https://rest.uniprot.org".
        """
        self._API_URL = api_url
        self._POLLING_INTERVAL = pooling_interval
        self.retries = self._setup_retries(total_retries, backoff_factor)
        self.session = requests.Session()
        self._setup_session()
        self._re_next_link = re.compile(r'<(.+)>; rel="next"')

    def _setup_retries(self, total_retries, backoff_factor) -> None:
        return Retry(
            total=total_retries,
            backoff_factor=backoff_factor,
            status_forcelist=[500, 502, 503, 504],
        )

    def _setup_session(self) -> None:
        self.session.mount("https://", HTTPAdapter(max_retries=self.retries))

    def check_response(self, response):
        try:
            response.raise_for_status()
        except requests.HTTPError:
            print(response.json())
            raise

    def submit_id_mapping(self, from_db, to_db, ids):
        request = requests.post(
            f"{self._API_URL}/idmapping/run",
            data={"from": from_db, "to": to_db, "ids": ",".join(ids)},
        )
        self.check_response(request)
        return request.json()["jobId"]

    def get_next_link(self, headers):
        if "Link" in headers:
            match = self._re_next_link.match(headers["Link"])
            if match:
                return match.group(1)

    def check_id_mapping_ready(self, job_id, from_db, to_db):
        while True:
            request = self.session.get(f"{self._API_URL}/idmapping/status/{job_id}")
            self.check_response(request)
            j = request.json()
            if "jobStatus" in j:
                if j["jobStatus"] in ["RUNNING", "NEW"]:
                    print(f"Retrying in {self._POLLING_INTERVAL}s")
                    time.sleep(self._POLLING_INTERVAL)
                else:
                    raise Exception(j["jobStatus"])
            else:
                try:
                    ready = bool(j["results"] or j["failedIds"])
                except KeyError:
                    raise requests.RequestException(
                        f"Unexpected response from {from_db} to {to_db}.\n"
                        'request.json() missing "results" and "failedIds"'
                    )
                return ready

    def get_batch(self, batch_response, file_format, compressed):
        batch_url = self.get_next_link(batch_response.headers)
        while batch_url:
            batch_response = self.session.get(batch_url)
            batch_response.raise_for_status()
            yield decode_results(batch_response, file_format, compressed)
            batch_url = self.get_next_link(batch_response.headers)

    def combine_batches(self, all_results, batch_results, file_format):
        if file_format == "json":
            for key in ("results", "failedIds"):
                if key in batch_results and batch_results[key]:
                    all_results[key] += batch_results[key]
        elif file_format == "tsv":
            return all_results + batch_results[1:]
        else:
            return all_results + batch_results
        return all_results

    def get_id_mapping_results_link(self, job_id):
        url = f"{self._API_URL}/idmapping/details/{job_id}"
        request = self.session.get(url)
        self.check_response(request)
        return request.json()["redirectURL"]

    def get_id_mapping_results_stream(self, url):
        if "/stream/" not in url:
            url = url.replace("/results/", "/results/stream/")
        request = self.session.get(url)
        self.check_response(request)
        parsed = urlparse(url)
        query = parse_qs(parsed.query)
        file_format = query["format"][0] if "format" in query else "json"
        compressed = (
            query["compressed"][0].lower() == "true" if "compressed" in query else False
        )
        return decode_results(request, file_format, compressed)

==> uniprot_id_mapper-1.1.3-py3-none-any.whl: UniProtMapper/interface.py <==
"""Holds the implementation of the base class for the UniProt id-mapping REST API."""

import re
from abc import ABC

import requests
from requests.adapters import HTTPAdapter, Retry

from .utils import read_fields_table

"""
Several methods were either taken or adapted from the Python example for the
UniProt ID mapping RESTful API documentation. The original code can be found at:
https://www.uniprot.org/help/id_mapping#submitting-an-id-mapping-job
"""


class BaseUniProt(ABC):
    """Base class for the UniProt rest APIs covered by this package:

    - BaseUniProt -> ProtMapper (ID mapping API)
    - BaseUniProt -> ProtKB (UniProtKB API)
    """

    def __init__(
        self,
        pooling_interval: int = 3,
        total_retries: int = 5,
        backoff_factor: float = 0.25,
        api_url: str = "https://rest.uniprot.org",
    ) -> None:
        """Initialize the class. This will set up the session and retry mechanism.

        Args:
            pooling_interval: The interval in seconds between polling the API.
                Defaults to 3.
            total_retries: The total number of retries to attempt. Defaults to 5.
            backoff_factor: The backoff factor to use when retrying. Defaults to 0.25.
            api_url: The url for the REST API. Defaults to "https://rest.uniprot.org".
        """
        self._API_URL = api_url
        self._POLLING_INTERVAL = pooling_interval
        self.retries = self._setup_retries(total_retries, backoff_factor)
        self.session = requests.Session()
        self._setup_session()
        self._re_next_link = re.compile(r'<(.+)>; rel="next"')

    @property
    def fields_table(self) -> None:
        return read_fields_table()

    def _setup_retries(self, total_retries, backoff_factor) -> None:
        return Retry(
            total=total_retries,
            backoff_factor=backoff_factor,
            status_forcelist=[500, 502, 503, 504],
        )

    def _setup_session(self) -> None:
        self.session.mount("https://", HTTPAdapter(max_retries=self.retries))

    def check_response(self, response) -> None:
        try:
            response.raise_for_status()
        except requests.HTTPError:
            print(response.json())
            raise

    def get_next_link(self, headers) -> str:
        if "Link" in headers:
            match = self._re_next_link.match(headers["Link"])
            if match:
                return match.group(1)
